@@ -29,13 +29,15 @@ use autodie;
 
 use File::Basename;
 use List::MoreUtils qw(any first_index);
-use Moo;
 use Text::ParseWords qw(shellwords);
 
 use Lintian::Data;
 use Lintian::Util qw(internal_error lstrip rstrip);
 
-with('Lintian::Check');
+use Moo;
+use namespace::clean;
+
+with 'Lintian::Check';
 
 has timers => (is => 'rwp', default => sub{ [] });
 
@@ -273,6 +275,18 @@ sub check_systemd_service_file {
         my $is_oneshot =any { /^oneshot$/ }
         $self->extract_service_file_values($file, 'Service', 'Type', 1);
 
+        # We are a "standalone" service file if we have no .path or .timer
+        # equivalent.
+        my $is_standalone = 1;
+        if ($file =~ m,lib/systemd/system/([^/]*?)@?\.service,) {
+            my $service = $1;
+            for my $x (qw(path timer)) {
+                $is_standalone = 0
+                  if $info->index_resolved_path(
+                    "lib/systemd/system/${service}.${x}");
+            }
+        }
+
         foreach my $target (@wanted_by) {
             $self->tag(
                 'systemd-service-file-refers-to-unusual-wantedby-target',
@@ -283,14 +297,17 @@ sub check_systemd_service_file {
         $self->tag('systemd-service-file-missing-documentation-key', $file,)
           unless $self->extract_service_file_values($file, 'Unit',
             'Documentation',1);
+
         $self->tag('systemd-service-file-missing-install-key', $file,)
           unless @wanted_by
           or
           $self->extract_service_file_values($file, 'Install', 'RequiredBy',1)
           or $self->extract_service_file_values($file, 'Install', 'Also',1)
           or $is_oneshot
+          or $is_standalone
           or $file !~ m,^lib/systemd/[^\/]+/[^\/]+\.service$,
           or $file =~ m,@\.service$,;
+
         my @pidfile
           = $self->extract_service_file_values($file,'Service','PIDFile',1);
         foreach my $x (@pidfile) {
@@ -440,20 +457,21 @@ sub check_maintainer_scripts {
 
     my $info = $self->info;
 
-    open(my $fd, '<', $info->lab_data_path('control-scripts'));
+    # get maintainer scripts
+    my %control = %{$self->info->control_scripts};
+
+    for my $file (keys %control) {
 
-    while (<$fd>) {
-        m/^(\S*) (.*)$/
-          or internal_error("bad line in control-scripts file: $_");
-        my $interpreter = $1;
-        my $file = $2;
         my $path = $info->control_index_resolved_path($file);
+        my $interpreter = $control{$file};
 
         # Don't follow unsafe links
-        next if not $path or not $path->is_open_ok;
-        # Don't try to parse the file if it does not appear to be a
-        # shell script
-        next if $interpreter !~ m/sh\b/;
+        next
+          unless $path && $path->is_open_ok;
+
+        # skip anything but shell scripts
+        next
+          unless $interpreter =~ m/sh\b/;
 
         my $sfd = $path->open;
         while (<$sfd>) {
@@ -469,7 +487,6 @@ sub check_maintainer_scripts {
         close($sfd);
     }
 
-    close($fd);
     return;
 }
 
